@@ -50,6 +50,7 @@
 #include "manconfig.h"
 #include "lib/error.h"
 #include "manp.h"
+#include "convert_name.h"
 
 static __inline__ void gripe_converting_name (const char *name)
 {
@@ -77,6 +78,7 @@ char *convert_name (const char *name, const char *alternate)
 #endif /* COMP_SRC */
 
 #ifdef COMP_CAT
+	/* TODO: BSD layout requires .0. */
 	to_name = strappend (NULL, name, "." COMPRESS_EXT, NULL);
 #else /* !COMP_CAT */
 	to_name = xstrdup (name);
@@ -91,12 +93,16 @@ char *convert_name (const char *name, const char *alternate)
 			gripe_converting_name (name);
 
 		*t1 = '/';
+		/* TODO: This is broken beyond repair. Rewrite this whole
+		 * function, after understanding what on earth it's trying
+		 * to achieve!
+		 */
 		*(t2 + 1) = 'c';
 		*(t2 + 3) = 't';
 	} else
 		gripe_converting_name (name);
 
-	if (0) /* (debug) */
+	if (debug)
 		fprintf (stderr, "to_name in convert_name () is %s\n", to_name);
 
 #ifdef COMP_SRC
