@@ -83,6 +83,10 @@ extern char *getwd();
 #  include <pwd.h>
 #endif /* SECURE_MAN_UID */
 
+#ifdef HAVE_LIBGEN_H
+#  include <libgen.h>
+#endif /* HAVE_LIBGEN_H */
+
 #ifdef HAVE_GETOPT_H
 #  include <getopt.h>
 #else /* !HAVE_GETOPT_H */
@@ -98,6 +102,7 @@ extern char *getwd();
 #include "lib/error.h"
 #include "lib/cleanup.h"
 #include "check_mandirs.h"
+#include "filenames.h"
 #include "manp.h"
 #include "security.h"
 
@@ -110,23 +115,27 @@ char *manp;
 char *database;
 extern char *extension;		/* for globbing.c */
 extern int force_rescan;	/* for check_mandirs.c */
+static char *single_filename = NULL;
+extern char *user_config_file;	/* for manp.c */
 
 /* default options */
 static const struct option long_options[] =
 {
-    {"create", 		no_argument,	0, 'c'},
-    {"debug",		no_argument, 	0, 'd'},
-    {"help",		no_argument,	0, 'h'},
-    {"no-purge",	no_argument,	0, 'p'},
-    {"quiet",		no_argument, 	0, 'q'},
-    {"user-db",		no_argument, 	0, 'u'},
-    {"no-straycats",    no_argument,	0, 's'},
-    {"test",		no_argument, 	0, 't'},
-    {"version",		no_argument, 	0, 'V'},
+    {"create", 		no_argument,		0, 'c'},
+    {"debug",		no_argument,		0, 'd'},
+    {"filename",	required_argument,	0, 'f'},
+    {"help",		no_argument,		0, 'h'},
+    {"no-purge",	no_argument,		0, 'p'},
+    {"quiet",		no_argument,		0, 'q'},
+    {"user-db",		no_argument,		0, 'u'},
+    {"no-straycats",    no_argument,		0, 's'},
+    {"test",		no_argument,		0, 't'},
+    {"config-file",	required_argument,	0, 'C'},
+    {"version",		no_argument,		0, 'V'},
     {0, 0, 0, 0}
 };
 
-static const char args[] = "cdhpqstuV";
+static const char args[] = "cdf:hpqstuC:V";
 static int check_for_strays = 1;
 static int purge = 1;
 static int user;
@@ -150,9 +159,6 @@ static char *xtmpfile;
 #ifdef SECURE_MAN_UID
 extern uid_t ruid;
 extern uid_t euid;
-#ifdef MAN_DB_UPDATES
-#  define DO_CHOWN
-#endif /* MAN_DB_UPDATES */
 #endif /* SECURE_MAN_UID */
 
 extern char *optarg;
@@ -162,7 +168,8 @@ extern int pages;
 
 static void usage (int status)
 {
-	printf (_("usage: %s [-dqspuc|-h|-V] [manpath]\n"), program_name);
+	printf (_("usage: %s [-dqspuct|-h|-V] [-C file] [-f filename] [manpath]\n"),
+		program_name);
 	printf (_(
 		"-d, --debug                 produce debugging info.\n"
 		"-q, --quiet                 work quietly, except for 'bogus' warning.\n"
@@ -170,6 +177,9 @@ static void usage (int status)
 		"-p, --no-purge              don't purge obsolete entries from the dbs.\n"
 		"-u, --user-db               produce user databases only.\n"
 		"-c, --create                create dbs from scratch, rather than updating.\n"
+		"-t, --test                  check manual pages for correctness.\n"
+		"-f, --filename              update just the entry for this filename.\n"
+		"-C, --config-file           use this user configuration file.\n"
 		"-V, --version               show version.\n"
 		"-h, --help                  show this usage message.\n")
 	);
@@ -193,29 +203,52 @@ static __inline__ void xrename (const char *from, const char *to)
 	}
 }
 
-/* CPhipps 2000/02/24 - Copy a file.
- * Still plenty of error handling could be added here. */
-static __inline__ void xcopy (const char *from, const char *to)
+/* CPhipps 2000/02/24 - Copy a file. */
+static int xcopy (const char *from, const char *to)
 {
-	FILE* ifp = fopen (from, "r");
-	FILE* ofp = fopen (to, "w");
+	FILE *ifp, *ofp;
+	int ret = 0;
 
-	if (!ifp || !ofp) {
-		if (ifp) fclose (ifp);
-		if (ofp) fclose (ofp);
-		if (errno != ENOENT)
-			perror ("fopen");
-		return;
+	ifp = fopen (from, "r");
+	if (!ifp) {
+		ret = -errno;
+		if (errno == ENOENT)
+			return 0;
+		perror ("fopen");
+		return ret;
+	}
+
+	ofp = fopen (to, "w");
+	if (!ofp) {
+		ret = -errno;
+		perror ("fopen");
+		fclose (ifp);
+		return ret;
 	}
 
 	while (!feof (ifp) && !ferror (ifp)) {
 		char buf[1024];
 		size_t in = fread (buf, 1, sizeof (buf), ifp);
-		if (in > 0)
-			fwrite (buf, 1, in, ofp);
+		if (in > 0) {
+			if (fwrite (buf, 1, in, ofp) == 0 && ferror (ofp)) {
+				ret = -errno;
+				error (0, errno, _("can't write to %s"), to);
+				break;
+			}
+		} else if (ferror (ifp)) {
+			ret = -errno;
+			error (0, errno, _("can't read from %s"), from);
+			break;
+		}
 	}
-	fclose(ifp);
-	fclose(ofp);
+
+	fclose (ifp);
+	fclose (ofp);
+
+	if (ret < 0)
+		xremove (to);
+
+	return ret;
 }
 
 /* chmod() with error checking */
@@ -246,7 +279,7 @@ static __inline__ void finish_up (void)
 #endif /* NDBM */
 }
 
-#ifdef DO_CHOWN
+#ifdef SECURE_MAN_UID
 /* chown() with error checking */
 static __inline__ void xchown (const char *path, uid_t owner, uid_t group)
 {
@@ -270,13 +303,38 @@ static __inline__ void do_chown (uid_t uid)
 	xchown (xfile, uid, -1);
 #  endif /* NDBM */
 }
-#endif /* DO_CHOWN */
+#endif /* SECURE_MAN_UID */
+
+/* Update a single file in an existing database. */
+static short update_one_file (const char *manpath, const char *filename)
+{
+	dbf = MYDBM_RWOPEN (database);
+	if (dbf) {
+		struct mandata info;
+		char *manpage;
+
+		memset (&info, 0, sizeof (struct mandata));
+		manpage = filename_info (filename, &info, "");
+		if (info.name) {
+			dbdelete (info.name, &info);
+			purge_pointers (info.name);
+		}
+
+		test_manfile (filename, manpath);
+	}
+	MYDBM_CLOSE (dbf);
+
+	return 1;
+}
 
 /* dont actually create any dbs, just do an update */
 static __inline__ short update_db_wrapper (const char *manpath)
 {
 	short amount;
 
+	if (single_filename)
+		return update_one_file (manpath, single_filename);
+
 	amount = update_db (manpath);
 	if (amount != EOF)
 		return amount;
@@ -287,6 +345,8 @@ static __inline__ short update_db_wrapper (const char *manpath)
 /* remove incomplete databases */
 static void cleanup (void *dummy)
 {
+	dummy = dummy; /* not used */
+
 #ifdef NDBM
 #  ifdef BERKELEY_DB
 	unlink (tmpdbfile);
@@ -320,7 +380,8 @@ static short mandb (const char *catpath, const char *manpath)
 		xremove (tmpdbfile);
 		amount = create_db (manpath);
 	} else {
-		xcopy (dbfile, tmpdbfile);
+		if (xcopy (dbfile, tmpdbfile) < 0)
+			return 0;
 		amount = update_db_wrapper (manpath);
 	}
 #  else /* !BERKELEY_DB NDBM */
@@ -333,8 +394,10 @@ static short mandb (const char *catpath, const char *manpath)
 		xremove (tmppagfile);
 		amount = create_db (manpath);
 	} else {
-		xcopy (dirfile, tmpdirfile);
-		xcopy (pagfile, tmppagfile);
+		if (xcopy (dirfile, tmpdirfile) < 0)
+			return 0;
+		if (xcopy (pagfile, tmppagfile) < 0)
+			return 0;
 		amount = update_db_wrapper (manpath);
 	}
 #  endif /* BERKELEY_DB NDBM */
@@ -345,7 +408,8 @@ static short mandb (const char *catpath, const char *manpath)
 		xremove (xtmpfile);
 		amount = create_db (manpath);
 	} else {
-		xcopy (xfile, xtmpfile);
+		if (xcopy (xfile, xtmpfile) < 0)
+			return 0;
 		amount = update_db_wrapper (manpath);
 	}
 #endif /* NDBM */
@@ -353,7 +417,7 @@ static short mandb (const char *catpath, const char *manpath)
 	return amount;
 }
 
-int main(int argc, char *argv[])
+int main (int argc, char *argv[])
 {
 	int c;
 	char *sys_manp;
@@ -362,7 +426,6 @@ int main(int argc, char *argv[])
 	int purged = 0;
 	int quiet_temp = 0;
 	char **mp;
-	char *locale;
 
 	int option_index; /* not used, but required by getopt_long() */
 
@@ -371,29 +434,23 @@ int main(int argc, char *argv[])
 	char *cwd = wd;
 #endif /* __profile__ */
 
-#ifdef DO_CHOWN
+#ifdef SECURE_MAN_UID
 	struct passwd *man_owner;
 #endif
 
 	program_name = xstrdup (basename (argv[0]));
 
 	/* initialise the locale */
-	locale = setlocale (LC_ALL, "");
-	if (locale)
-		locale = xstrdup (locale);
-	else {
+	if (!setlocale (LC_ALL, ""))
 		/* Obviously can't translate this. */
 		error (0, 0, "can't set the locale; make sure $LC_* and $LANG "
 			     "are correct");
-		locale = "C";
-	}
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
 	while ((c = getopt_long (argc, argv, args,
 				 long_options, &option_index)) != EOF) {
 		switch (c) {
-
 			case 'd':
 				debug = 1;
 				break;
@@ -416,14 +473,23 @@ int main(int argc, char *argv[])
 			case 't':
 				opt_test = 1;
 				break;
+			case 'f':
+				single_filename = optarg;
+				create = 0;
+				purge = 0;
+				check_for_strays = 0;
+				break;
+			case 'C':
+				user_config_file = optarg;
+				break;
 			case 'V':
-				ver();
+				ver ();
 				break;
 			case 'h':
-				usage(OK);
+				usage (OK);
 				break;
 			default:
-				usage(FAIL);
+				usage (FAIL);
 				break;
 		}
 	}
@@ -440,7 +506,7 @@ int main(int argc, char *argv[])
 	init_security ();
 #endif /* SECURE_MAN_UID */
 
-#ifdef DO_CHOWN
+#ifdef SECURE_MAN_UID
 	man_owner = getpwnam (MAN_OWNER);
 	if (man_owner == NULL)
 		error (FAIL, 0,
@@ -448,7 +514,7 @@ int main(int argc, char *argv[])
 		       MAN_OWNER);
 	if (!user && euid != 0 && euid != man_owner->pw_uid)
 		user = 1;
-#endif /* DO_CHOWN */
+#endif /* SECURE_MAN_UID */
 
 
 	/* This is required for get_catpath(), regardless */
@@ -506,18 +572,23 @@ int main(int argc, char *argv[])
 		force_rescan = 0;
 		if (purge) {
 			database = mkdbname (catpath);
-			purged += purge_missing (*mp);
+			purged += purge_missing (*mp, catpath);
 		}
 
 		push_cleanup (cleanup, NULL);
-		amount += mandb (catpath, *mp);
+		if (single_filename) {
+			if (STRNEQ (*mp, single_filename, strlen (*mp)))
+				amount += mandb (catpath, *mp);
+			/* otherwise try the next manpath */
+		} else
+			amount += mandb (catpath, *mp);
 
 		if (!opt_test) {
 			finish_up ();
-#ifdef DO_CHOWN
+#ifdef SECURE_MAN_UID
 			if (global_manpath && euid == 0)
 				do_chown (man_owner->pw_uid);
-#endif
+#endif /* SECURE_MAN_UID */
 		} else
 			cleanup (NULL);
 		pop_cleanup ();
@@ -556,5 +627,7 @@ int main(int argc, char *argv[])
 		chdir (cwd);
 #endif /* __profile__ */
 
+	if (!amount)
+		error (FAIL, 0, _("No databases updated."));
 	exit (OK);
 }
