@@ -108,7 +108,8 @@ struct list {
 
 static struct list *namestore, *tailstore;
 
-#define SECTION		-4
+#define SECTION		-5
+#define DEFINE_USER	-4
 #define DEFINE		-3
 #define MANDB_MAP_USER	-2
 #define MANDB_MAP	-1
@@ -119,13 +120,15 @@ static struct list *namestore, *tailstore;
 char *tmplist[MAXDIRS];
 char *manpathlist[MAXDIRS];
 
+char *user_config_file = NULL;
+
 static void mkcatdirs (const char *mandir, const char *catdir);
 static __inline__ char *get_manpath (char *path);
 static __inline__ char *has_mandir (const char *p);
 static __inline__ char *fsstnd (const char *path);
 static char *def_path (int flag);
 static void add_dir_to_list (char **lp, const char *dir);
-static char **add_dir_to_path_list (char **mp, const char *p);
+static char **add_dir_to_path_list (char **mphead, char **mp, const char *p);
 
 
 static void add_to_list (const char *key, const char *cont, int flag)
@@ -142,7 +145,7 @@ static void add_to_list (const char *key, const char *cont, int flag)
 		namestore = list;
 }
 
-static char *get_from_list (char *key, int flag)
+static const char *get_from_list (const char *key, int flag)
 {
 	struct list *list;
 
@@ -164,12 +167,23 @@ static struct list *iterate_over_list (struct list *prev, char *key, int flag)
 	return NULL;
 }
 
-char *get_def (char *thing, char *def)
+/* Must not return DEFINEs set in ~/.manpath. This is used to fetch
+ * definitions used in raised-privilege code; if in doubt, be conservative!
+ */
+const char *get_def (const char *thing, const char *def)
 {
-	char *config_def = get_from_list (thing, DEFINE);
+	const char *config_def = get_from_list (thing, DEFINE);
 	return config_def ? config_def : def;
 }
-	
+
+const char *get_def_user (const char *thing, const char *def)
+{
+	const char *config_def = get_from_list (thing, DEFINE_USER);
+	if (!config_def)
+		config_def = get_from_list (thing, DEFINE);
+	return config_def ? config_def : def;
+}
+
 static void print_list (void)
 {
 	struct list *list;
@@ -195,11 +209,11 @@ static void add_sections (char *sections)
 	}
 }
 
-char **get_sections (void)
+const char **get_sections (void)
 {
 	struct list *list;
 	int length = 0;
-	char **sections, **sectionp;
+	const char **sections, **sectionp;
 
 	for (list = namestore; list; list = list->next)
 		if (list->flag == SECTION)
@@ -213,9 +227,10 @@ char **get_sections (void)
 	return sections;
 }
 
-static void add_def (char *thing, char *config_def, int flag)
+static void add_def (char *thing, char *config_def, int flag, int user)
 {
-	add_to_list (thing, flag == 2 ? config_def : "", DEFINE);
+	add_to_list (thing, flag == 2 ? config_def : "",
+		     user ? DEFINE_USER : DEFINE);
 
 	if (debug)
 		fprintf (stderr, "Defined `%s' as `%s'.\n", thing, config_def);
@@ -328,11 +343,9 @@ static char *pathappend (char *oldpath, const char *appendage)
 
 static __inline__ void gripe_reading_mp_config (const char *file)
 {
-	if (!quiet)
-		error (FAIL, 0,
-		       _("can't make sense of the manpath configuration file "
-			 "%s"),
-		       file);
+	error (FAIL, 0,
+	       _("can't make sense of the manpath configuration file %s"),
+	       file);
 }
 
 static __inline__ void gripe_stat_file (const char *file)
@@ -347,12 +360,17 @@ static __inline__ void gripe_not_directory (const char *dir)
 		error (0, 0, _("warning: %s isn't a directory"), dir);
 }
 
+static void gripe_overlong_list (void)
+{
+	error (FAIL, 0, _("manpath list too long"));
+}
 
 /* accept a manpath list, separated with ':', return the associated 
    catpath list */
 char *cat_manpath (char *manp)
 {
-	char *catp = NULL, *path, *catdir;
+	char *catp = NULL;
+	const char *path, *catdir;
 
 	for (path = strsep (&manp, ":"); path; path = strsep (&manp, ":")) {
 		catdir = get_from_list (path, MANDB_MAP_USER);
@@ -549,7 +567,7 @@ static char *add_system_manpath (const char *systems, const char *manpathlist)
  * $HOME/bin in his path and the directory $HOME/bin/man exists -- the
  * directory $HOME/bin/man will be added to the manpath.
  */
-char *guess_manpath (const char *systems)
+static char *guess_manpath (const char *systems)
 {
 	char *path = getenv ("PATH");
 
@@ -644,8 +662,7 @@ extern uid_t euid;			/* initial effective user id */
 static void
 mkcatdirs (const char *mandir, const char *catdir)
 {
-	char manname[PATH_MAX+6];
-	char catname[PATH_MAX+6];
+	char *manname, *catname;
 #ifdef SECURE_MAN_UID
 	struct passwd *man_owner = get_man_owner ();
 #endif
@@ -674,7 +691,8 @@ mkcatdirs (const char *mandir, const char *catdir)
 			drop_effective_privs ();
 		}
 		/* then the hierarchy */
-		sprintf (catname, "%s/cat1", catdir);
+		catname = strappend (NULL, catdir, "/cat1", NULL);
+		manname = strappend (NULL, mandir, "/man1", NULL);
 		if (is_directory (catdir) == 1) {
 			int j;
 			regain_effective_privs ();
@@ -683,8 +701,8 @@ mkcatdirs (const char *mandir, const char *catdir)
 					 "creating catdir hierarchy %s	",
 					 catdir);
 			for (j = 1; j <= 9; j++) {
-				sprintf (catname, "%s/cat%d", catdir, j);
-				sprintf (manname, "%s/man%d", mandir, j);
+				catname[strlen (catname) - 1] = '0' + j;
+				manname[strlen (manname) - 1] = '0' + j;
 				if ((is_directory (manname) == 1)
 				 && (is_directory (catname) != 1)) {
 					if (mkdir (catname,
@@ -706,6 +724,8 @@ mkcatdirs (const char *mandir, const char *catdir)
 				fprintf (stderr, "\n");
 			drop_effective_privs ();
 		}
+		free (catname);
+		free (manname);
 		umask (oldmask);
 	}
 }
@@ -724,21 +744,26 @@ static void add_to_dirlist (FILE *config, int user)
 		while (isspace (*bp))
 			bp++;
 
+		/* TODO: would like a (limited) replacement for sscanf()
+		 * here that allocates its own memory. At that point check
+		 * everything that sprintf()s manpath et al!
+		 */
 		if (*bp == '#' || *bp == '\0')
 			continue;
 		else if (strncmp (bp, "NO", 2) == 0)
-			continue;	/* mach any word starting with NO */
+			continue;	/* match any word starting with NO */
 		else if (sscanf (bp, "MANBIN %*s") == 1)
 			continue;
-		else if (sscanf (bp, "MANDATORY_MANPATH %s", key) == 1)
+		else if (sscanf (bp, "MANDATORY_MANPATH %49s", key) == 1)
 			add_mandatory (key);	
-		else if (sscanf (bp, "MANPATH_MAP %s %s", key, cont) == 2) 
+		else if (sscanf (bp, "MANPATH_MAP %49s %511s", key, cont) == 2) 
 			add_manpath_map (key, cont);
-		else if ((c = sscanf (bp, "MANDB_MAP %s %s", key, cont)) > 0) 
+		else if ((c = sscanf (bp, "MANDB_MAP %49s %511s",
+				      key, cont)) > 0) 
 			add_mandb_map (key, cont, c, user);
-		else if ((c = sscanf (bp, "DEFINE %50s %511[^\n]",
+		else if ((c = sscanf (bp, "DEFINE %49s %511[^\n]",
 				      key, cont)) > 0)
-			add_def (key, cont, c);
+			add_def (key, cont, c, user);
 		else if (sscanf (bp, "SECTION %511[^\n]", cont) == 1)
 			add_sections (cont);
 		else if (sscanf (bp, "SECTIONS %511[^\n]", cont) == 1)
@@ -758,7 +783,11 @@ void read_config_file(void)
 
 	home = xstrdup (getenv ("HOME"));
 	if (home) {
-		char *dotmanpath = strappend (home, "/.manpath", NULL);
+		char *dotmanpath;
+		if (!user_config_file)
+			dotmanpath = strappend (home, "/.manpath", NULL);
+		else
+			dotmanpath = xstrdup (user_config_file);
 		config = fopen (dotmanpath, "r");
 		if (config != NULL) {
 			if (debug)
@@ -933,8 +962,11 @@ static __inline__ char *get_manpath (char *path)
 static void add_dir_to_list (char **lp, const char *dir)
 {
 	int status;
+	int pos = 0;
 
 	while (*lp != NULL) {
+		if (pos > MAXDIRS - 1)
+			gripe_overlong_list ();
 		if (!strcmp (*lp, dir)) {
 			if (debug)
 				fprintf (stderr,
@@ -943,6 +975,7 @@ static void add_dir_to_list (char **lp, const char *dir)
 			return;
 		}
 		lp++;
+		pos++;
 	}
 
 	/* Not found -- add it. */
@@ -992,12 +1025,15 @@ static __inline__ char *has_mandir (const char *path)
 	return NULL;
 }
 
-static __inline__ char **add_dir_to_path_list (char **mp, const char *p)
+static char **add_dir_to_path_list (char **mphead, char **mp, const char *p)
 {
 	int status;
 	char wd[PATH_MAX];
 	char *cwd = wd;
 
+	if (mp - mphead > MAXDIRS - 1)
+		gripe_overlong_list ();
+
 	status = is_directory (p);
 
 	if (status < 0)
@@ -1027,15 +1063,17 @@ static __inline__ char **add_dir_to_path_list (char **mp, const char *p)
 void create_pathlist (const char *manp, char **mp)
 {
 	const char *p, *end;
+	char **mphead = mp;
 
 	/* Expand the manpath into a list for easier handling. */
 
 	for (p = manp;; p = end + 1) {
 		end = strchr (p, ':');
 		if (end)
-			mp = add_dir_to_path_list (mp, xstrndup (p, end - p));
+			mp = add_dir_to_path_list (mphead, mp,
+						   xstrndup (p, end - p));
 		else {
-			mp = add_dir_to_path_list (mp, p);
+			mp = add_dir_to_path_list (mphead, mp, p);
 			break;
 		}
 	}
