@@ -36,12 +36,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
+#include <assert.h>
 
 #ifndef STDC_HEADERS
 extern int errno;
 #endif
 
-#ifndef S_ISLNK /* loosing sys/stat.h */
+#ifndef S_ISLNK /* losing sys/stat.h */
 #  if defined(S_IFMT) && defined(S_IFLNK)
 #    define S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
 #  endif
@@ -62,8 +63,6 @@ extern int errno;
 #  endif /* HAVE_NDIR_H */
 #endif /* HAVE_DIRENT_H  */
 
-extern char *realpath();
-
 #if defined(STDC_HEADERS)
 #  include <string.h>
 #  include <stdlib.h>
@@ -73,6 +72,7 @@ extern char *realpath();
 #  include <strings.h>
 #else /* no string(s) header */
 extern char *strrchr();
+extern char *realpath();
 #endif /* no string(s) header */
 
 #if defined(HAVE_UNISTD_H)
@@ -105,31 +105,34 @@ extern char *strrchr();
 #include "security.h"
 #include "ult_src.h"
 
-/* Find minimum value hard link filename for given file and inode */
-static __inline__ char *ult_hardlink (char *fullpath, long inode)
+/* Find minimum value hard link filename for given file and inode.
+ * Returns a newly allocated string.
+ */
+static char *ult_hardlink (const char *fullpath, ino_t inode)
 {
 	DIR *mdir;
 	struct dirent *manlist;
-	char link[PATH_MAX];
-	char dir[PATH_MAX];
-	char *t;
+	char *link, *dir, *ret;
+	const char *slash;
 
-	t = strrchr (fullpath, '/');
-	*t = '\0';
-	(void) strcpy (dir, fullpath);
-	*t = '/';
-	(void) strcpy (link, ++t);
+	slash = strrchr (fullpath, '/');
+	assert (slash);
+	dir = xstrndup (fullpath, slash - fullpath);
+	link = xstrdup (++slash);
 
 	mdir = opendir (dir);
 	if (mdir == NULL) {
 		error (0, errno, _("can't search directory %s"), dir);
+		free (dir);
+		free (link);
 		return NULL;
 	}
 
 	while ((manlist = readdir (mdir))) {
 		if (manlist->d_ino == inode &&
 		    strcmp (link, manlist->d_name) > 0) {
-		  	(void) strcpy (link, manlist->d_name);
+			free (link);
+			link = xstrdup (manlist->d_name);
 			if (debug)
 				fprintf (stderr, "ult_hardlink: (%s)\n", link);
 		}
@@ -139,16 +142,23 @@ static __inline__ char *ult_hardlink (char *fullpath, long inode)
 	/* If we already are the link with the smallest name value */
 	/* return NULL */
 
-	if (strcmp (link, t) == 0)
+	if (strcmp (link, slash) == 0) {
+		free (dir);
+		free (link);
 		return NULL;
+	}
 
-	return strcpy (strrchr (fullpath, '/') + 1, link);
+	ret = strappend (NULL, dir, "/", link, NULL);
+	free (dir);
+	free (link);
+	return ret;
 }
 
 #ifdef S_ISLNK
-/* use realpath() to resolve all sym links within 'fullpath'. 'mantree' is 
-   the man hierarchy */
-static __inline__ char *ult_softlink (char *fullpath, const char *mantree)
+/* Use realpath() to resolve all sym links within 'fullpath'.
+ * Returns a newly allocated string.
+ */
+static char *ult_softlink (const char *fullpath)
 {
 	char resolved_path[PATH_MAX];
 
@@ -165,13 +175,15 @@ static __inline__ char *ult_softlink (char *fullpath, const char *mantree)
 	if (debug)
 		fprintf (stderr, "ult_softlink: (%s)\n", resolved_path);
 
-	return strcpy (fullpath, resolved_path);
+	return xstrdup (resolved_path);
 }
 #endif /* S_ISLNK */
 
-/* test `buffer' to see if it contains a .so include, if so and it's not an 
-   absolute filename, copy it into `basename' at `rel' and return 1 */
-static __inline__ int test_for_include (char *buffer, char *rel)
+/* Test 'buffer' to see if it contains a .so include. If so and it's not an 
+ * absolute filename, return newly allocated string whose contents are the
+ * include.
+ */
+static char *test_for_include (const char *buffer)
 {
 	/* strip out any leading whitespace (if any) */
 	while (isspace ((int) *buffer))
@@ -196,15 +208,13 @@ static __inline__ int test_for_include (char *buffer, char *rel)
 		 * follow any absolute inclusions in our quest for the 
 		 * ultimate source file */
 		if (*buffer != '/') {
-			/* copy filename into rel address */
-			while (*buffer && !isspace ((int) *buffer))
-				*(rel++) = *(buffer++);
-
-			*rel = '\0';
-			return 1;
+			const char *end = buffer;
+			while (*end && !isspace (*end))
+				++end;
+			return xstrndup (buffer, end - buffer);
 		}
 	}
-	return 0;
+	return NULL;
 }
 
 /*
@@ -215,22 +225,23 @@ static __inline__ int test_for_include (char *buffer, char *rel)
  * name is full pathname, path is the MANPATH directory (/usr/man)
  * flags is a combination of SO_LINK | SOFT_LINK | HARD_LINK
  */
-char *ult_src (const char *name, const char *path, struct stat *buf, int flags)
+const char *ult_src (const char *name, const char *path,
+		     struct stat *buf, int flags)
 {
-	static char basename[2048];	/* must be static */
+	static char *basename;		/* must be static */
 	static short recurse; 		/* must be static */
-	static char *relative; 		/* must be static */
 
 	/* initialise the function */
 
 	/* as ult_softlink() & ult_hardlink() do all of their respective
 	 * resolving in one call, only need to sort them out once
 	 */
-	   
+
 	if (recurse == 0) {
 		struct stat new_buf;
-		(void) strcpy (basename, name);
-		relative = basename + strlen (path) + 1;
+		if (basename)
+			free (basename);
+		basename = xstrdup (name);
 
 		if (debug)
 			fprintf (stderr, "\nult_src: File %s in mantree %s\n",
@@ -249,19 +260,29 @@ char *ult_src (const char *name, const char *path, struct stat *buf, int flags)
 #ifdef S_ISLNK
 		/* Permit semi local (inter-tree) soft links */
 		if (flags & SOFT_LINK) {
-			if (S_ISLNK (buf->st_mode))
+			if (S_ISLNK (buf->st_mode)) {
 				/* Is a symlink, resolve it. */
-			/*	(void) ult_softlink (basename, path); */
-				if (!ult_softlink (basename, path))
+				char *softlink = ult_softlink (basename);
+				if (softlink) {
+					free (basename);
+					basename = softlink;
+				} else
 					return NULL;
+			}
 		}
 #endif /* S_ISLNK */
 
 		/* Only deal with local (inter-dir) HARD links */
 		if (flags & HARD_LINK) {
-			if (buf->st_nlink > 1)
+			if (buf->st_nlink > 1) {
 				/* Has HARD links, find least value */
-				(void) ult_hardlink (basename, buf->st_ino); 
+				char *hardlink = ult_hardlink (basename,
+							       buf->st_ino);
+				if (hardlink) {
+					free (basename);
+					basename = hardlink;
+				}
+			}
 		}
 	}
 
@@ -273,7 +294,6 @@ char *ult_src (const char *name, const char *path, struct stat *buf, int flags)
 
 	if (flags & SO_LINK) {
 		char buffer[1024], *bptr;
-		int val;
 		FILE *fp;
 #ifdef COMP_SRC
 		struct compression *comp;
@@ -298,8 +318,8 @@ char *ult_src (const char *name, const char *path, struct stat *buf, int flags)
 				free (comp->file);
 				if (!filename)
 					return NULL;
-				(void) strcat (basename, ".");
-				(void) strcat (basename, comp->ext);
+				basename = strappend (basename, ".", comp->ext,
+						      NULL);
 				drop_effective_privs ();
 				fp = fopen (filename, "r");
 				regain_effective_privs ();
@@ -321,37 +341,35 @@ char *ult_src (const char *name, const char *path, struct stat *buf, int flags)
 		/* make sure that we skip over any comments */
 		do {
 			bptr = fgets (buffer, 1024, fp);
-		} while (bptr && strncmp (buffer, ".\\\"", 3) == 0);
+		} while (bptr && STRNEQ (buffer, ".\\\"", 3));
 
 		fclose(fp);
 
 		if (buffer) {
-			/* Restore the original path from before
-			 * ult_softlink() etc., in case it went outside the
-			 * mantree.
-			 */
-			char *basename_copy = xstrdup (basename);
-			(void) strcpy (basename, path);
-			(void) strcat (basename, "/");
-			val = test_for_include (buffer, relative);
-			if (!val)
-				(void) strcpy (basename, basename_copy);
-			free (basename_copy);
-		} else
-			val = EOF;
+			char *include = test_for_include (buffer);
+			if (include) {
+				const char *ult;
 
-		if (val == 1) {			/* keep on looking... */
-			char *ult;
+				/* Restore the original path from before
+				 * ult_softlink() etc., in case it went
+				 * outside the mantree.
+				 */
+				free (basename);
+				basename = strappend (NULL, path, "/", include,
+						      NULL);
+				free (include);
 
-			if (debug)
-				fprintf (stderr, "ult_src: points to %s\n",
-					 basename);
+				if (debug)
+					fprintf (stderr,
+						 "ult_src: points to %s\n",
+						 basename);
 
-			recurse++;
-			ult = ult_src (basename, path, NULL, flags);
-			recurse--;
+				recurse++;
+				ult = ult_src (basename, path, NULL, flags);
+				recurse--;
 
-			return ult;
+				return ult;
+			}
 		}
 	}
 
