@@ -25,5 +25,5 @@
 #define SOFT_LINK	0002
 #define HARD_LINK	0004
 
-extern char *ult_src (const char *name, const char *path,
-		      struct stat *buf, int flags);
+extern const char *ult_src (const char *name, const char *path,
+			    struct stat *buf, int flags);
